@@ -153,7 +153,6 @@ done:
 
 static NV_STATUS test_unexpected_completed_values(uvm_va_space_t *va_space)
 {
-    NV_STATUS status;
     uvm_gpu_t *gpu;
 
     for_each_va_space_gpu(gpu, va_space) {
@@ -747,14 +746,14 @@ static NvU32 get_available_gpfifo_entries(uvm_channel_t *channel)
 {
     NvU32 pending_entries;
 
-    uvm_spin_lock(&channel->pool->lock);
+    uvm_channel_pool_lock(channel->pool);
 
     if (channel->cpu_put >= channel->gpu_get)
         pending_entries = channel->cpu_put - channel->gpu_get;
     else
         pending_entries = channel->cpu_put + channel->num_gpfifo_entries - channel->gpu_get;
 
-    uvm_spin_unlock(&channel->pool->lock);
+    uvm_channel_pool_unlock(channel->pool);
 
     return channel->num_gpfifo_entries - pending_entries - 1;
 }
