@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2022 NVIDIA Corporation
+    Copyright (c) 2022-2023 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -50,10 +50,10 @@ typedef enum
 //
 // A policy covers one or more contiguous Linux VMAs or portion of a VMA and
 // does not cover non-existant VMAs.
-// The VA range is determined from either the uvm_va_range_t for UVM managed
+// The VA range is determined from either the uvm_va_range_t for managed
 // allocations or the uvm_va_policy_node_t for HMM allocations.
 //
-typedef struct uvm_va_policy_struct
+struct uvm_va_policy_struct
 {
     // Read duplication policy for this VA range (unset, enabled, or disabled).
     uvm_read_duplication_policy_t read_duplication;
@@ -66,7 +66,7 @@ typedef struct uvm_va_policy_struct
     // their page tables updated to access the (possibly remote) pages.
     uvm_processor_mask_t accessed_by;
 
-} uvm_va_policy_t;
+};
 
 // Policy nodes are used for storing policies in HMM va_blocks.
 // The va_block lock protects the tree so that invalidation callbacks can
@@ -94,6 +94,12 @@ bool uvm_va_policy_is_read_duplicate(uvm_va_policy_t *policy, uvm_va_space_t *va
 // Locking: The va_block lock must be held.
 uvm_va_policy_t *uvm_va_policy_get(uvm_va_block_t *va_block, NvU64 addr);
 
+// Return a uvm_va_policy_node_t given a uvm_va_policy_t pointer.
+static uvm_va_policy_node_t *uvm_va_policy_node_from_policy(uvm_va_policy_t *policy)
+{
+    return container_of(policy, uvm_va_policy_node_t, policy);
+}
+
 #if UVM_IS_CONFIG_HMM()
 
 // Module load/exit
@@ -239,6 +245,11 @@ static NV_STATUS uvm_va_policy_set_range(uvm_va_block_t *va_block,
     return NV_OK;
 }
 
+static uvm_va_policy_node_t *uvm_va_policy_node_iter_first(uvm_va_block_t *va_block, NvU64 start, NvU64 end)
+{
+    return NULL;
+}
+
 #endif // UVM_IS_CONFIG_HMM()
 
 #endif // __UVM_VA_POLICY_H__
