@@ -642,6 +642,12 @@ nvswitch_os_get_platform_time
     void
 );
 
+NvU64
+nvswitch_os_get_platform_time_epoch
+(
+    void
+);
+
 #if (defined(_WIN32) || defined(_WIN64))
 #define NVSWITCH_PRINT_ATTRIB(str, arg1)
 #else
@@ -802,6 +808,14 @@ nvswitch_os_strncmp
     NvLength length
 );
 
+char*
+nvswitch_os_strncat
+(
+    char *s1,
+    const char *s2,
+    NvLength length
+);
+
 void *
 nvswitch_os_memset
 (
