@@ -203,6 +203,8 @@ namespace DisplayPort
 
         virtual bool getSDPExtnForColorimetrySupported() = 0;
 
+        virtual bool getPanelFwRevision(NvU16 *revision) = 0;
+
         virtual bool getIgnoreMSACap() = 0;
 
         virtual AuxRetry::status setIgnoreMSAEnable(bool msaTimingParamIgnoreEn) = 0;
@@ -211,6 +213,10 @@ namespace DisplayPort
 
         virtual NvBool isDSCSupported() = 0;
 
+        virtual NvBool isDSCDecompressionSupported() = 0;
+
+        virtual NvBool isDSCPassThroughSupported() = 0;
+
         virtual DscCaps getDscCaps() = 0;
 
         //
@@ -262,6 +268,7 @@ namespace DisplayPort
 
         virtual bool setPanelReplayConfig(panelReplayConfig prcfg) = 0;
         virtual bool isPanelReplaySupported() = 0;
+        virtual bool getPanelReplayStatus(PanelReplayStatus *pPrStatus) = 0;
 
     protected:
             virtual ~Device() {}
