@@ -381,6 +381,8 @@ namespace DisplayPort
 
         virtual bool getSDPExtnForColorimetrySupported();
 
+        virtual bool getPanelFwRevision(NvU16 *revision);
+
         virtual bool isPowerSuspended();
 
         virtual void setPanelPowerParams(bool bSinkPowerStateD0, bool bPanelPowerStateOn);
@@ -439,11 +441,13 @@ namespace DisplayPort
         bool isPanelReplaySupported(void);
         void getPanelReplayCaps(void);
         bool setPanelReplayConfig(panelReplayConfig prcfg);
+        bool getPanelReplayStatus(PanelReplayStatus *pPrStatus);
 
         NvBool getDSCSupport();
         bool getFECSupport();
         NvBool isDSCPassThroughSupported();
         NvBool isDSCSupported();
+        NvBool isDSCDecompressionSupported();
         NvBool isDSCPossible();
         bool isFECSupported();
         bool readAndParseDSCCaps();
@@ -451,6 +455,7 @@ namespace DisplayPort
         bool parseDscCaps(const NvU8 *buffer, NvU32 bufferSize);
         bool parseBranchSpecificDscCaps(const NvU8 *buffer, NvU32 bufferSize);
         bool setDscEnable(bool enable);
+        bool setDscEnableDPToHDMIPCON(bool bDscEnable, bool bEnablePassThroughForPCON);
         bool getDscEnable(bool *pEnable);
         unsigned getDscVersionMajor();
         unsigned getDscVersionMinor();
