@@ -59,16 +59,18 @@
 
 #define NV_DP_REGKEY_FORCE_EDP_ILR                    "DP_BYPASS_EDP_ILR_REV_CHECK"
 
+// Regkey to enable OUI caching/restoring in release branch.
+#define NV_DP_REGKEY_ENABLE_OUI_RESTORING             "DP_ENABLE_OUI_RESTORING"
+
+// Message to power down video stream before power down link (set D3)
+#define NV_DP_REGKEY_POWER_DOWN_PHY                   "DP_POWER_DOWN_PHY"
+
 //
 // DSC capability of downstream device should be decided based on device's own
 // and its parent's DSC capability.
 //
 #define NV_DP_DSC_MST_CAP_BUG_3143315                  "DP_DSC_MST_CAP_BUG_3143315"
 
-#define NV_DP_REGKEY_NO_REPLY_TIMER_FOR_BUSY_WAITING   "NO_REPLY_TIMER_FOR_BUSY_WAITING"
-
-#define NV_DP_REGKEY_DPCD_PROBING_FOR_BUSY_WAITING     "DP_DPCD_PROBING_FOR_BUSY_WAITING"
-
 //
 // Data Base used to store all the regkey values.
 // The actual data base is declared statically in dp_evoadapter.cpp.
@@ -100,8 +102,8 @@ struct DP_REGKEY_DATABASE
     bool  bOptLinkKeptAliveSst;
     bool  bBypassEDPRevCheck;
     bool  bDscMstCapBug3143315;
-    bool  bNoReplyTimerForBusyWaiting;
-    bool  bDpcdProbingForBusyWaiting;
+    bool  bEnableOuiRestoring;
+    bool  bPowerDownPhyBeforeD3;
 };
 
 #endif //INCLUDED_DP_REGKEYDATABASE_H
