@@ -920,23 +920,31 @@ void DeviceImpl::applyOUIOverrides()
             if ((buffer[3] == 0x53) && (buffer[4] == 0x59) && (buffer[5] == 0x4E) && (buffer[6] == 0x41))
             {
                 // For Synaptic VMM5331 and VMM5320, it only support MSA-Over-MST for DP after Firmware 5.4.5
-                if (buffer[7] == 0x53 &&
-                    (buffer[8] == 0x31 || buffer[8] == 0x20))
+                if (buffer[7] == 0x53)
                 {
-                    this->bSdpExtCapable = False;
+                    //
+                    // This flag will be checked only in DSC Pass through cases (MST).
+                    // All Synaptics VMM53XX chips which support pass through can only support
+                    // color formats that are listed in 0x69h even in pass through mode.
+                    //
                     this->bDscPassThroughColorFormatWar = true;
-
-                    //
-                    // Check firmware version
-                    // 0x50A: FW/SW Major Revision.
-                    // 0x50B: FW/SW Minor Revision.
-                    // 0x50C: Build Number.
-                    //
-                    if ((buffer[10] >= 0x06) ||
-                        ((buffer[10] == 0x05) && (buffer[11] >= 0x05)) ||
-                        ((buffer[10] == 0x05) && (buffer[11] == 0x04) && (buffer[12] >= 0x05)))
+            
+                    if ((buffer[8] == 0x31) || (buffer[8] == 0x20))
                     {
-                        this->bSdpExtCapable = True;
+                        this->bSdpExtCapable = False;
+
+                        //
+                        // Check firmware version
+                        // 0x50A: FW/SW Major Revision.
+                        // 0x50B: FW/SW Minor Revision.
+                        // 0x50C: Build Number.
+                        //
+                        if ((buffer[10] >= 0x06) ||
+                            ((buffer[10] == 0x05) && (buffer[11] >= 0x05)) ||
+                            ((buffer[10] == 0x05) && (buffer[11] == 0x04) && (buffer[12] >= 0x05)))
+                        {
+                            this->bSdpExtCapable = True;
+                        }
                     }
                 }
             }
@@ -1043,6 +1051,46 @@ bool DeviceImpl::getSDPExtnForColorimetrySupported()
     return (this->bSdpExtCapable == True);
 }
 
+bool DeviceImpl::getPanelFwRevision(NvU16 *revision)
+{
+    NvU8 fwRevisionMajor   = 0;
+    NvU8 fwRevisionMinor   = 0;
+    unsigned size          = 0;
+    unsigned nakReason     = NakUndefined;
+
+    if (!revision)
+    {
+        return false;
+    }
+
+    *revision = 0;
+
+    //
+    // On faked mux devices, we cannot check if the device has
+    // the capability as we don't have access to aux.
+    //
+    if (this->isFakedMuxDevice())
+    {
+        return false;
+    }
+
+    if (AuxBus::success != this->getDpcdData(NV_DPCD14_FW_SW_REVISION_MAJOR,
+                                             &fwRevisionMajor, sizeof(fwRevisionMajor), &size, &nakReason))
+    {
+        return false;
+    }
+
+    if (AuxBus::success != this->getDpcdData(NV_DPCD14_FW_SW_REVISION_MINOR,
+                                             &fwRevisionMinor, sizeof(fwRevisionMinor), &size, &nakReason))
+    {
+        return false;
+    }
+
+    *revision = (fwRevisionMajor << 8) | fwRevisionMinor;
+
+    return true;
+}
+
 bool DeviceImpl::isPowerSuspended()
 {
     bool bPanelPowerOn, bDPCDPowerStateD0;
@@ -1460,7 +1508,11 @@ NvBool DeviceImpl::getDSCSupport()
     {
         if (FLD_TEST_DRF(_DPCD14, _DSC_SUPPORT, _DSC_SUPPORT, _YES, byte))
         {
-            dscCaps.bDSCSupported = true;
+            dscCaps.bDSCDecompressionSupported = true;
+        }
+        if (FLD_TEST_DRF(_DPCD20, _DSC_SUPPORT, _PASS_THROUGH_SUPPORT, _YES, byte))
+        {
+            dscCaps.bDSCPassThroughSupported = true;
         }
     }
 
@@ -1469,6 +1521,11 @@ NvBool DeviceImpl::getDSCSupport()
         DP_LOG(("DP-DEV> DSC Support AUX READ failed for %s!", address.toString(sb)));
     }
 
+    if (dscCaps.bDSCDecompressionSupported || dscCaps.bDSCPassThroughSupported)
+    {
+        dscCaps.bDSCSupported = true;
+    }
+
     return dscCaps.bDSCSupported;
 }
 
@@ -1517,6 +1574,47 @@ bool DeviceImpl::setPanelReplayConfig(panelReplayConfig prcfg)
     return false;
 }
 
+bool DeviceImpl::getPanelReplayStatus(PanelReplayStatus *pPrStatus)
+{
+    NvU8 state = 0;
+    unsigned size = 0;
+    unsigned nakReason = NakUndefined;
+
+    if (pPrStatus == NULL)
+    {
+        DP_ASSERT(0);
+        return false;
+    }
+
+    if(AuxBus::success == this->getDpcdData(NV_DPCD20_PANEL_REPLAY_AND_FRAME_LOCK_STATUS,
+        &state, sizeof(state), &size, &nakReason))
+    {
+        switch (DRF_VAL(_DPCD20, _PANEL_REPLAY_AND_FRAME_LOCK_STATUS, _PR_STATUS, state))
+        {
+            case NV_DPCD20_PANEL_REPLAY_AND_FRAME_LOCK_STATUS_PR_STATUS_STATE_0:
+            pPrStatus->prState = PanelReplay_Inactive;
+            break;
+
+            case NV_DPCD20_PANEL_REPLAY_AND_FRAME_LOCK_STATUS_PR_STATUS_STATE_1:
+            pPrStatus->prState = PanelReplay_CaptureAndDisplay;
+            break;
+
+            case NV_DPCD20_PANEL_REPLAY_AND_FRAME_LOCK_STATUS_PR_STATUS_STATE_2:
+            pPrStatus->prState = PanelReplay_DisplayFromRfb;
+            break;
+
+            default:
+            pPrStatus->prState = PanelReplay_Undefined;
+            break;
+        }
+        return true;
+    }
+    else
+    {    
+        return false;
+    }
+}
+
 bool DeviceImpl::getFECSupport()
 {
     NvU8 byte          = 0;
@@ -1547,6 +1645,11 @@ NvBool DeviceImpl::isDSCSupported()
     return dscCaps.bDSCSupported;
 }
 
+NvBool DeviceImpl::isDSCDecompressionSupported()
+{
+    return dscCaps.bDSCDecompressionSupported;
+}
+
 NvBool DeviceImpl::isDSCPassThroughSupported()
 {
     return dscCaps.bDSCPassThroughSupported;
@@ -1885,7 +1988,7 @@ void DeviceImpl::setDscDecompressionDevice(bool bDscCapBasedOnParent)
                         this->devDoingDscDecompression = this;
                         this->bDSCPossible = true;
                     }
-                    else if (this->parent->isDSCSupported())
+                    else if (this->parent->isDSCDecompressionSupported())
                     {
                         //
                         // This condition takes care of DSC capable sink devices 
@@ -1898,12 +2001,15 @@ void DeviceImpl::setDscDecompressionDevice(bool bDscCapBasedOnParent)
                 }
                 else
                 {
-                    // This condition takes care of branch device capable of DSC.
-                    this->devDoingDscDecompression = this;
-                    this->bDSCPossible = true;
+                    if (this->isDSCDecompressionSupported())
+                    {
+                        // This condition takes care of branch device capable of DSC decoding.
+                        this->devDoingDscDecompression = this;
+                        this->bDSCPossible = true;
+                    }
                 }
-            }           
-            else if (this->parent && this->parent->isDSCSupported())
+            }
+            else if (this->parent && this->parent->isDSCDecompressionSupported())
             {
                 //
                 // This condition takes care of sink devices not capable of DSC 
@@ -1916,7 +2022,7 @@ void DeviceImpl::setDscDecompressionDevice(bool bDscCapBasedOnParent)
     }
     else
     {
-        if (this->isDSCSupported())
+        if (this->isDSCDecompressionSupported())
         {
             this->bDSCPossible = true;
             this->devDoingDscDecompression = this;
@@ -2043,6 +2149,54 @@ bool DeviceImpl::setDscEnable(bool enable)
     }
 }
 
+
+
+bool DeviceImpl::setDscEnableDPToHDMIPCON(bool bDscEnable, bool bEnablePassThroughForPCON)
+{
+    NvU8 dscEnableByte              = 0; 
+    unsigned size                   = 0;
+    unsigned nakReason              = NakUndefined;
+    AuxBus::status dscEnableStatus  = AuxBus::success;
+    Address::StringBuffer buffer;
+    DP_USED(buffer);
+
+    if (!this->isDSCPossible())
+    {
+        DP_LOG(("DP-DEV> DSC is not supported on DP to HDMI PCON - %s"));
+        return false;
+    }
+
+    if (bDscEnable)
+    {
+        if(bEnablePassThroughForPCON)
+        {
+            dscEnableByte = FLD_SET_DRF(_DPCD20, _DSC_PASS_THROUGH, _ENABLE, _YES, dscEnableByte);
+            DP_LOG(("DP-DEV> Enabling DSC Pass through on DP to HDMI PCON device - %s",
+                    this->getTopologyAddress().toString(buffer)));
+        }
+        else
+        {
+            dscEnableByte = FLD_SET_DRF(_DPCD14, _DSC_ENABLE, _SINK, _YES, dscEnableByte);
+            DP_LOG(("DP-DEV> Enabling DSC decompression on DP to HDMI PCON device - %s",
+                    this->getTopologyAddress().toString(buffer)));
+        }
+
+    }
+
+    dscEnableStatus = this->setDpcdData(NV_DPCD14_DSC_ENABLE,
+                              &dscEnableByte, sizeof dscEnableByte, &size, &nakReason);
+
+    if (dscEnableStatus != AuxBus::success)
+    {
+        DP_LOG(("DP-DEV> Setting DSC Enable on DP to HDMI PCON %s failed", 
+                this->getTopologyAddress().toString(buffer)));
+        return false;
+ 
+    }
+
+    return true;
+}
+
 unsigned DeviceImpl::getDscVersionMajor()
 {
     return dscCaps.versionMajor;
