@@ -74,7 +74,8 @@ NvBool nvGetCursorImageSurfaces(
                 nvEvoGetSurfaceFromHandle(pDevEvo,
                                           pOpenDevSurfaceHandles,
                                           pParams->surfaceHandle[eye],
-                                          NV_EVO_CHANNEL_MASK_CURSOR_ALL);
+                                          TRUE /* isUsedByCursorChannel */,
+                                          FALSE /* isUsedByLayerChannel */);
             if ((pSurfaceEvo == NULL) ||
                 (pSurfaceEvo->isoType != NVKMS_MEMORY_ISO)) {
                 return FALSE;
@@ -160,12 +161,8 @@ FlipCursorImage(NVDispEvoPtr pDispEvo,
 
     pFlipRequest = &pFlipParams->request;
 
-    pFlipRequest->sd[sd].head[head] = (struct NvKmsFlipCommonParams) {
-        .cursor = {
-            .image          = *pImageParams,
-            .imageSpecified = TRUE,
-        },
-    };
+    pFlipRequest->sd[sd].head[head].cursor.image = *pImageParams;
+    pFlipRequest->sd[sd].head[head].cursor.imageSpecified = TRUE;
 
     pFlipRequest->sd[sd].requestedHeadsBitMask = NVBIT(head);
 
