@@ -58,7 +58,6 @@ enum NvKmsAllocDeviceStatus nvAssignEvoCaps(NVDevEvoPtr pDevEvo)
 {
 #define ENTRY(_classPrefix,                                               \
               _pEvoHal,                                                   \
-              _supportsInbandStereoSignaling,                             \
               _supportsDP13,                                              \
               _supportsHDMI20,                                            \
               _inputLutAppliesToBase,                                     \
@@ -82,8 +81,6 @@ enum NvKmsAllocDeviceStatus nvAssignEvoCaps(NVDevEvoPtr pDevEvo)
         },                                                                \
         .evoCaps = {                                                      \
             .supportsDP13              = _supportsDP13,                   \
-            .supportsInbandStereoSignaling =                              \
-                _supportsInbandStereoSignaling,                           \
             .supportsHDMI20            = _supportsHDMI20,                 \
             .validNIsoFormatMask       = _validNIsoFormatMask,            \
             .inputLutAppliesToBase     = _inputLutAppliesToBase,          \
@@ -176,24 +173,23 @@ enum NvKmsAllocDeviceStatus nvAssignEvoCaps(NVDevEvoPtr pDevEvo)
         const NVEvoCapsRec evoCaps;
     } dispTable[] = {
         /*
-         * genericPageKind------------------------+
-         * inputLutAppliesToBase ------------+    |
-         * supportsHDMI20 ----------------+  |    |
-         * supportsDP13 ---------------+  |  |    |
-         * inbandStereoSignaling----+  |  |  |    |
-         * pEvoHal --------------+  |  |  |  |    |
-         * windowClassPrefix     |  |  |  |  |    |
-         * classPrefix |         |  |  |  |  |    |
-         *         |   |         |  |  |  |  |    |
+         * genericPageKind---------------------+
+         * inputLutAppliesToBase ---------+    |
+         * supportsHDMI20 -------------+  |    |
+         * supportsDP13 ------------+  |  |    |
+         * pEvoHal --------------+  |  |  |    |
+         * windowClassPrefix     |  |  |  |    |
+         * classPrefix |         |  |  |  |    |
+         *         |   |         |  |  |  |    |
          */
-        ENTRY_NVD(C7, C6, &nvEvoC6, 1, 1, 1, 0, TURING_GENERIC_KIND),
-        ENTRY_NVD(C6, C6, &nvEvoC6, 1, 1, 1, 0, TURING_GENERIC_KIND),
-        ENTRY_NVD(C5, C5, &nvEvoC5, 1, 1, 1, 0, TURING_GENERIC_KIND),
-        ENTRY_NVD(C3, C3, &nvEvoC3, 1, 1, 1, 0, FERMI_GENERIC_KIND),
-        ENTRY_EVO(98,     &nvEvo94, 1, 1, 1, 1, FERMI_GENERIC_KIND),
-        ENTRY_EVO(97,     &nvEvo94, 1, 1, 1, 1, FERMI_GENERIC_KIND),
-        ENTRY_EVO(95,     &nvEvo94, 1, 0, 1, 1, FERMI_GENERIC_KIND),
-        ENTRY_EVO(94,     &nvEvo94, 1, 0, 0, 1, FERMI_GENERIC_KIND),
+        ENTRY_NVD(C7, C6, &nvEvoC6, 1, 1, 0, TURING_GENERIC_KIND),
+        ENTRY_NVD(C6, C6, &nvEvoC6, 1, 1, 0, TURING_GENERIC_KIND),
+        ENTRY_NVD(C5, C5, &nvEvoC5, 1, 1, 0, TURING_GENERIC_KIND),
+        ENTRY_NVD(C3, C3, &nvEvoC3, 1, 1, 0, FERMI_GENERIC_KIND),
+        ENTRY_EVO(98,     &nvEvo94, 1, 1, 1, FERMI_GENERIC_KIND),
+        ENTRY_EVO(97,     &nvEvo94, 1, 1, 1, FERMI_GENERIC_KIND),
+        ENTRY_EVO(95,     &nvEvo94, 0, 1, 1, FERMI_GENERIC_KIND),
+        ENTRY_EVO(94,     &nvEvo94, 0, 0, 1, FERMI_GENERIC_KIND),
     };
 
     int i;
