@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1999-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -46,6 +46,8 @@
 #include <class/cl003e.h> // NV01_MEMORY_SYSTEM
 #include <class/cl0071.h> // NV01_MEMORY_SYSTEM_OS_DESCRIPTOR
 
+#include <ctrl/ctrl00fd.h>
+
 #define NV_CTL_DEVICE_ONLY(nv)                 \
 {                                              \
     if (((nv)->flags & NV_FLAG_CONTROL) == 0)  \
@@ -64,6 +66,56 @@
     }                                          \
 }
 
+static NvBool RmIsDeviceRefNeeded(NVOS54_PARAMETERS *pApi)
+{
+    switch(pApi->cmd)
+    {
+        case NV00FD_CTRL_CMD_ATTACH_MEM:
+            return NV_TRUE;
+        default:
+            return NV_FALSE;
+    }
+}
+
+static NV_STATUS RmGetDeviceFd(NVOS54_PARAMETERS *pApi, NvS32 *pFd)
+{
+    RMAPI_PARAM_COPY paramCopy;
+    void *pKernelParams;
+    NvU32 paramSize;
+    NV_STATUS status;
+
+    *pFd = -1;
+
+    switch(pApi->cmd)
+    {
+        case NV00FD_CTRL_CMD_ATTACH_MEM:
+            paramSize = sizeof(NV00FD_CTRL_ATTACH_MEM_PARAMS);
+            break;
+        default:
+            return NV_ERR_INVALID_ARGUMENT;
+    }
+
+    RMAPI_PARAM_COPY_INIT(paramCopy, pKernelParams, pApi->params, paramSize, 1);
+
+    status = rmapiParamsAcquire(&paramCopy, NV_TRUE);
+    if (status != NV_OK)
+        return status;
+
+    switch(pApi->cmd)
+    {
+        case NV00FD_CTRL_CMD_ATTACH_MEM:
+            *pFd = (NvS32)((NV00FD_CTRL_ATTACH_MEM_PARAMS *)pKernelParams)->devDescriptor;
+            break;
+        default:
+            NV_ASSERT(0);
+            break;
+    }
+
+    NV_ASSERT(rmapiParamsRelease(&paramCopy) == NV_OK);
+
+    return status;
+}
+
 // Only return errors through pApi->status
 static void RmCreateOsDescriptor(NVOS32_PARAMETERS *pApi, API_SECURITY_INFO secInfo)
 {
@@ -91,6 +143,7 @@ static void RmCreateOsDescriptor(NVOS32_PARAMETERS *pApi, API_SECURITY_INFO secI
     pageCount = (1 + ((allocSize - 1) / os_page_size));
 
     writable = FLD_TEST_DRF(OS32, _ATTR2, _PROTECTION_USER, _READ_WRITE, pApi->data.AllocOsDesc.attr2);
+
     flags = FLD_SET_DRF_NUM(_LOCK_USER_PAGES, _FLAGS, _WRITE, writable, flags);
     rmStatus = os_lock_user_pages(pDescriptor, pageCount, &pPageArray, flags);
     if (rmStatus == NV_OK)
@@ -243,6 +296,7 @@ NV_STATUS RmIoctl(
     secInfo.privLevel = osIsAdministrator() ? RS_PRIV_LEVEL_USER_ROOT : RS_PRIV_LEVEL_USER;
     secInfo.paramLocation = PARAM_LOCATION_USER;
     secInfo.pProcessToken = NULL;
+    secInfo.gpuOsInfo = NULL;
     secInfo.clientOSInfo = nvfp->ctl_nvfp;
     if (secInfo.clientOSInfo == NULL)
         secInfo.clientOSInfo = nvfp;
@@ -282,7 +336,7 @@ NV_STATUS RmIoctl(
                     (!FLD_TEST_DRF(OS02, _FLAGS, _MAPPING, _NO_MAP, flags)) &&
                     (pParms->status == NV_OK))
                 {
-                    if (rm_create_mmap_context(nv, pParms->hRoot,
+                    if (rm_create_mmap_context(pParms->hRoot,
                             pParms->hObjectParent, pParms->hObjectNew,
                             pParms->pMemory, pParms->limit + 1, 0,
                             NV_MEMORY_DEFAULT,
@@ -464,7 +518,7 @@ NV_STATUS RmIoctl(
 
             if (pParms->status == NV_OK)
             {
-                pParms->status = rm_create_mmap_context(nv, pParms->hClient,
+                pParms->status = rm_create_mmap_context(pParms->hClient,
                                  pParms->hDevice, pParms->hMemory,
                                  pParms->pLinearAddress, pParms->length,
                                  pParms->offset,
@@ -700,6 +754,9 @@ NV_STATUS RmIoctl(
         case NV_ESC_RM_CONTROL:
         {
             NVOS54_PARAMETERS *pApi = data;
+            void *priv = NULL;
+            nv_file_private_t *dev_nvfp = NULL;
+            NvS32 fd;
 
             NV_CTL_DEVICE_ONLY(nv);
 
@@ -709,7 +766,51 @@ NV_STATUS RmIoctl(
                 goto done;
             }
 
+            if (RmIsDeviceRefNeeded(pApi))
+            {
+                rmStatus = RmGetDeviceFd(pApi, &fd);
+                if (rmStatus != NV_OK)
+                {
+                    goto done;
+                }
+
+                dev_nvfp = nv_get_file_private(fd, NV_FALSE, &priv);
+                if (dev_nvfp == NULL)
+                {
+                    rmStatus = NV_ERR_INVALID_DEVICE;
+                    goto done;
+                }
+
+                // Check to avoid cyclic dependency with NV_ESC_REGISTER_FD
+                if (!portAtomicCompareAndSwapU32(&dev_nvfp->register_or_refcount,
+                                                 NVFP_TYPE_REFCOUNTED,
+                                                 NVFP_TYPE_NONE))
+                {
+                    // Is this already refcounted...
+                    if (dev_nvfp->register_or_refcount != NVFP_TYPE_REFCOUNTED)
+                    {
+                        nv_put_file_private(priv);
+                        rmStatus = NV_ERR_IN_USE;
+                        goto done;
+                    }
+                }
+
+                secInfo.gpuOsInfo = priv;
+            }
+
             Nv04ControlWithSecInfo(pApi, secInfo);
+
+            if ((pApi->status != NV_OK) && (priv != NULL))
+            {
+                //
+                // No need to reset `register_or_refcount` as it might be set
+                // for previous successful calls. We let it clear with FD close.
+                //
+                nv_put_file_private(priv);
+
+                secInfo.gpuOsInfo = NULL;
+            }
+
             break;
         }
 
@@ -751,7 +852,7 @@ NV_STATUS RmIoctl(
             }
 
             // LOCK: acquire API lock
-            rmStatus = rmApiLockAcquire(API_LOCK_FLAGS_NONE, RM_LOCK_MODULES_OSAPI);
+            rmStatus = rmapiLockAcquire(API_LOCK_FLAGS_NONE, RM_LOCK_MODULES_OSAPI);
             if (rmStatus != NV_OK)
                 goto done;
 
@@ -759,7 +860,7 @@ NV_STATUS RmIoctl(
             if (nvfp->ctl_nvfp != NULL)
             {
                 // UNLOCK: release API lock
-                rmApiLockRelease();
+                rmapiLockRelease();
                 rmStatus = NV_ERR_INVALID_STATE;
                 goto done;
             }
@@ -776,7 +877,7 @@ NV_STATUS RmIoctl(
             if (ctl_nvfp == NULL)
             {
                 // UNLOCK: release API lock
-                rmApiLockRelease();
+                rmapiLockRelease();
                 rmStatus = NV_ERR_INVALID_ARGUMENT;
                 goto done;
             }
@@ -787,11 +888,23 @@ NV_STATUS RmIoctl(
             {
                 nv_put_file_private(priv);
                 // UNLOCK: release API lock
-                rmApiLockRelease();
+                rmapiLockRelease();
                 rmStatus = NV_ERR_INVALID_ARGUMENT;
                 goto done;
             }
 
+            // Check to avoid cyclic dependency with device refcounting
+            if (!portAtomicCompareAndSwapU32(&nvfp->register_or_refcount,
+                                             NVFP_TYPE_REGISTERED,
+                                             NVFP_TYPE_NONE))
+            {
+                nv_put_file_private(priv);
+                // UNLOCK: release API lock
+                rmapiLockRelease();
+                rmStatus = NV_ERR_IN_USE;
+                goto done;
+            }
+
             //
             // nvfp->ctl_nvfp is read outside the lock, so set it atomically.
             // Note that once set, this can never be removed until the fd
@@ -803,7 +916,7 @@ NV_STATUS RmIoctl(
             nvfp->ctl_nvfp_priv = priv;
 
             // UNLOCK: release API lock
-            rmApiLockRelease();
+            rmapiLockRelease();
 
             // NOTE: nv_put_file_private(priv) is not called here.  It MUST be
             // called during cleanup of this nvfp.
