@@ -146,9 +146,9 @@ static const GPUCHILDPRESENT gpuChildrenPresent_GA100[] =
     { classId(KernelDisplay), 1 },
     { classId(VirtMemAllocator), 1 },
     { classId(OBJDPAUX), 1 },
-    { classId(OBJFAN), 1 },
+    { classId(Fan), 1 },
     { classId(OBJHSHUBMANAGER), 1 },
-    { classId(OBJHSHUB), 2 },
+    { classId(Hshub), 2 },
     { classId(MemorySystem), 1 },
     { classId(KernelMemorySystem), 1 },
     { classId(MemoryManager), 1 },
@@ -235,9 +235,9 @@ static const GPUCHILDPRESENT gpuChildrenPresent_GA102[] =
     {classId(KernelDisplay), 1},
     {classId(VirtMemAllocator), 1},
     {classId(OBJDPAUX), 1},
-    {classId(OBJFAN), 1},
+    {classId(Fan), 1},
     {classId(OBJHSHUBMANAGER), 1 },
-    {classId(OBJHSHUB), 2 },
+    {classId(Hshub), 2 },
     {classId(MemorySystem), 1},
     {classId(KernelMemorySystem), 1},
     {classId(MemoryManager), 1},
