@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -21,6 +21,8 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#define NVOC_KERN_GMMU_H_PRIVATE_ACCESS_ALLOWED
+
 #include "gpu/mmu/kern_gmmu.h"
 #include "gpu/bus/kern_bus.h"
 
@@ -313,9 +315,11 @@ kgmmuGetHwPteApertureFromMemdesc_GM107
                 aperture = NV_MMU_PTE_APERTURE_SYSTEM_NON_COHERENT_MEMORY;
             }
             break;
-        case ADDR_FBMEM:
-        case ADDR_FABRIC:
         case ADDR_FABRIC_V2:
+        case ADDR_FABRIC_MC:
+            aperture = NV_MMU_PTE_APERTURE_PEER_MEMORY;
+            break;
+        case ADDR_FBMEM:
             aperture = NV_MMU_PTE_APERTURE_VIDEO_MEMORY;
             break;
         default:
