@@ -84,6 +84,7 @@ msencGetEngineDescFromAllocParams(OBJGPU *pGpu, NvU32 externalClassId, void *pAl
     {
         KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
         MIG_INSTANCE_REF ref;
+        RM_ENGINE_TYPE rmEngineType;
 
         NV_ASSERT_OK(
             kmigmgrGetInstanceRefFromClient(pGpu, pKernelMIGManager,
@@ -91,9 +92,9 @@ msencGetEngineDescFromAllocParams(OBJGPU *pGpu, NvU32 externalClassId, void *pAl
 
         NV_ASSERT_OK(
             kmigmgrGetLocalToGlobalEngineType(pGpu, pKernelMIGManager, ref,
-                                              NV2080_ENGINE_TYPE_NVENC(engineInstance),
-                                              &engineInstance));
-        return ENG_MSENC(NV2080_ENGINE_TYPE_NVENC_IDX(engineInstance));
+                                              RM_ENGINE_TYPE_NVENC(engineInstance),
+                                              &rmEngineType));
+        return ENG_MSENC(RM_ENGINE_TYPE_NVENC_IDX(rmEngineType));
     }
 
     // Get the right class as per engine instance.
