@@ -54,8 +54,6 @@
 #include "kernel/gpu/nvlink/kernel_nvlink.h"
 #include "ctrl/ctrl0041.h"
 
-#include "published/ampere/ga100/dev_mmu.h"
-
 static NvU32
 _memoryfabricMemDescGetNumAddr
 (
@@ -81,6 +79,214 @@ _memoryfabricMemDescGetNumAddr
     return (memdescGetSize(pMemDesc) / pageSize);
 }
 
+static NV_STATUS
+_memoryfabricValidatePhysMem
+(
+    NvHandle           hClient,
+    NvHandle           hPhysMem,
+    OBJGPU            *pOwnerGpu,
+    MEMORY_DESCRIPTOR **ppPhysMemDesc
+)
+{
+    RsResourceRef *pPhysmemRef;
+    MEMORY_DESCRIPTOR *pPhysMemDesc;
+    NvU32 physPageSize;
+    NV_STATUS status;
+
+    if (hPhysMem == 0)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Invalid physmem handle\n");
+
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+
+    status = serverutilGetResourceRef(hClient, hPhysMem, &pPhysmemRef);
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR,
+                  "Failed to get resource in resserv for physmem handle\n");
+
+        return status;
+    }
+
+    pPhysMemDesc = (dynamicCast(pPhysmemRef->pResource, Memory))->pMemDesc;
+
+    if ((memdescGetAddressSpace(pPhysMemDesc) != ADDR_FBMEM) ||
+        (pOwnerGpu != pPhysMemDesc->pGpu))
+    {
+        NV_PRINTF(LEVEL_ERROR, "Invalid physmem handle passed\n");
+
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+
+    physPageSize = memdescGetPageSize(pPhysMemDesc, AT_GPU);
+    if ((physPageSize != NV_MEMORY_FABRIC_PAGE_SIZE_2M) &&
+        (physPageSize != NV_MEMORY_FABRIC_PAGE_SIZE_512M))
+    {
+        NV_PRINTF(LEVEL_ERROR, "Physmem page size should be 2MB\n");
+
+        return NV_ERR_INVALID_ARGUMENT;
+    }
+
+    *ppPhysMemDesc = pPhysMemDesc;
+
+    return NV_OK;
+}
+
+static NV_STATUS
+_memoryFabricDetachMem
+(
+    MEMORY_DESCRIPTOR   *pFabricMemDesc,
+    NvU64                offset
+)
+{
+    NV_STATUS status;
+    RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
+    FABRIC_ATTCH_MEM_INFO_NODE *pAttachMemInfoNode;
+    NODE *pNode = NULL;
+    FABRIC_VASPACE *pFabricVAS;
+    MEMORY_DESCRIPTOR *pPhysMemDesc;
+    FABRIC_MEMDESC_DATA *pMemdescData;
+
+    pMemdescData = (FABRIC_MEMDESC_DATA *)memdescGetMemData(pFabricMemDesc);
+
+    status = btreeSearch(offset, &pNode, pMemdescData->pAttachMemInfoTree);
+    if (status != NV_OK)
+        return status;
+
+    pAttachMemInfoNode = (FABRIC_ATTCH_MEM_INFO_NODE *)pNode->Data;
+    pPhysMemDesc = pAttachMemInfoNode->pPhysMemDesc;
+    pFabricVAS = dynamicCast(pPhysMemDesc->pGpu->pFabricVAS, FABRIC_VASPACE);
+
+    fabricvaspaceUnmapPhysMemdesc(pFabricVAS, pFabricMemDesc, offset,
+                                  pPhysMemDesc,
+                                  pAttachMemInfoNode->physMapLength);
+
+    NV_ASSERT_OK(pRmApi->Free(pRmApi, pFabricVAS->hClient,
+                              pAttachMemInfoNode->hDupedPhysMem));
+
+    btreeUnlink(&pAttachMemInfoNode->node, &pMemdescData->pAttachMemInfoTree);
+
+    portMemFree(pAttachMemInfoNode);
+
+    return NV_OK;
+}
+
+static void
+_memoryFabricBatchDetachMem
+(
+    MEMORY_DESCRIPTOR *pFabricMemDesc
+)
+{
+    FABRIC_MEMDESC_DATA *pMemdescData;
+    NODE *pNode = NULL;
+    NvU64 offset;
+    pMemdescData = (FABRIC_MEMDESC_DATA *)memdescGetMemData(pFabricMemDesc);
+
+    if (pMemdescData == NULL)
+        return;
+
+    btreeEnumStart(0, &pNode, pMemdescData->pAttachMemInfoTree);
+    while (pNode != NULL)
+    {
+        offset = pNode->keyStart;
+        btreeEnumNext(&pNode, pMemdescData->pAttachMemInfoTree);
+        NV_ASSERT_OK(_memoryFabricDetachMem(pFabricMemDesc, offset));
+    }
+}
+
+static NV_STATUS
+_memoryFabricAttachMem
+(
+    MemoryFabric                *pMemoryFabric,
+    NV00F8_CTRL_ATTACH_MEM_INFO *pAttachInfo
+)
+{
+    NV_STATUS status;
+    Memory *pMemory = staticCast(pMemoryFabric, Memory);
+    OBJGPU *pGpu = pMemory->pGpu;
+    MEMORY_DESCRIPTOR *pPhysMemDesc;
+    MEMORY_DESCRIPTOR *pFabricMemDesc = pMemory->pMemDesc;
+    FABRIC_MEMDESC_DATA *pMemdescData;
+    NvHandle hDupedPhysMem = 0;
+    RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
+    FABRIC_VASPACE *pFabricVAS = dynamicCast(pGpu->pFabricVAS, FABRIC_VASPACE);
+    FABRIC_ATTCH_MEM_INFO_NODE *pNode;
+
+    pMemdescData = (FABRIC_MEMDESC_DATA *)memdescGetMemData(pFabricMemDesc);
+
+    if (!(pMemdescData->allocFlags & NV00F8_ALLOC_FLAGS_FLEXIBLE_FLA))
+    {
+        NV_PRINTF(LEVEL_ERROR, "Unsupported fabric memory type\n");
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
+    status = _memoryfabricValidatePhysMem(RES_GET_CLIENT_HANDLE(pMemory),
+                                          pAttachInfo->hMemory,
+                                          pGpu, &pPhysMemDesc);
+
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, status);
+
+    status = pRmApi->DupObject(pRmApi, pFabricVAS->hClient, pFabricVAS->hDevice,
+                               &hDupedPhysMem, RES_GET_CLIENT_HANDLE(pMemory),
+                               pAttachInfo->hMemory, 0);
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Failed to dup physmem handle\n");
+        return status;
+    }
+
+    status = fabricvaspaceMapPhysMemdesc(pFabricVAS,
+                                         pFabricMemDesc,
+                                         pAttachInfo->offset,
+                                         pPhysMemDesc,
+                                         pAttachInfo->mapOffset,
+                                         pAttachInfo->mapLength,
+                                         0);
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Failed to map FLA\n");
+        goto freeDupedMem;
+    }
+
+    pNode = portMemAllocNonPaged(sizeof(*pNode));
+    if (pNode == NULL)
+    {
+        status = NV_ERR_NO_MEMORY;
+        goto unmapVas;
+    }
+
+    portMemSet(pNode, 0, sizeof(*pNode));
+
+    pNode->node.keyStart = pAttachInfo->offset;
+    pNode->node.keyEnd   = pAttachInfo->offset;
+    pNode->physMapLength = pAttachInfo->mapLength;
+    pNode->pPhysMemDesc  = pPhysMemDesc;
+    pNode->hDupedPhysMem = hDupedPhysMem;
+    pNode->node.Data     = pNode;
+
+    status = btreeInsert(&pNode->node, &pMemdescData->pAttachMemInfoTree);
+    if (status != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Failed to track attach mem info\n");
+        goto freeNode;
+    }
+
+    return NV_OK;
+
+freeNode:
+    portMemFree(pNode);
+
+unmapVas:
+    fabricvaspaceUnmapPhysMemdesc(pFabricVAS, pFabricMemDesc, pAttachInfo->offset,
+                                  pPhysMemDesc, pAttachInfo->mapLength);
+
+freeDupedMem:
+    NV_ASSERT_OK(pRmApi->Free(pRmApi, pFabricVAS->hClient, hDupedPhysMem));
+
+    return status;
+}
+
 static void
 _memoryfabricMemDescDestroyCallback
 (
@@ -89,24 +295,28 @@ _memoryfabricMemDescDestroyCallback
     MEMORY_DESCRIPTOR *pMemDesc
 )
 {
-    RM_API *pRmApi   = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
-    FABRIC_VASPACE *pFabricVAS = dynamicCast(pGpu->pFabricVAS, FABRIC_VASPACE);
+    RM_API              *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
+    FABRIC_VASPACE      *pFabricVAS;
+    RmPhysAddr          *pteArray;
+    FABRIC_MEMDESC_DATA *pMemdescData;
+    NvU32                numAddr;
+    NvU32                pageSize;
 
-    FABRIC_MEMDESC_DATA *pMemdescData =
-                            (FABRIC_MEMDESC_DATA *)memdescGetMemData(pMemDesc);
-
-    RmPhysAddr *pteArray = memdescGetPteArrayForGpu(pMemDesc, pGpu,
-                                    VAS_ADDRESS_TRANSLATION(pGpu->pFabricVAS));
-
-    NvU32 numAddr = _memoryfabricMemDescGetNumAddr(pMemDesc);
+    NV_ASSERT_OR_RETURN_VOID(pGpu->pFabricVAS != NULL);
 
+    pFabricVAS = dynamicCast(pGpu->pFabricVAS, FABRIC_VASPACE);
+    pMemdescData = (FABRIC_MEMDESC_DATA *)memdescGetMemData(pMemDesc);
+    pteArray = memdescGetPteArrayForGpu(pMemDesc, pGpu, VAS_ADDRESS_TRANSLATION(pGpu->pFabricVAS));
+    numAddr = _memoryfabricMemDescGetNumAddr(pMemDesc);
     // Get the page size from the memory descriptor.
-    NvU32 pageSize = memdescGetPageSize(pMemDesc,
-                                    VAS_ADDRESS_TRANSLATION(pGpu->pFabricVAS));
+    pageSize = memdescGetPageSize(pMemDesc, VAS_ADDRESS_TRANSLATION(pGpu->pFabricVAS));
 
     // Remove the fabric memory allocations from the map.
     fabricvaspaceVaToGpaMapRemove(pFabricVAS, pteArray[0]);
 
+    // Detach any pending memory...
+    _memoryFabricBatchDetachMem(pMemDesc);
+
     if (!pFabricVAS->bRpcAlloc)
     {
         //
@@ -117,108 +327,21 @@ _memoryfabricMemDescDestroyCallback
         fabricvaspaceBatchFree(pFabricVAS, pteArray, numAddr, (pageSize >> RM_PAGE_SHIFT));
     }
 
-    // Destroy the duped physical video memory handle.
-    if ((pMemdescData != NULL) && (pMemdescData->hDupedVidmem != 0))
+    if (pMemdescData != NULL)
     {
-        NV_ASSERT(pRmApi->Free(pRmApi, pFabricVAS->hClient,
-                               pMemdescData->hDupedVidmem) == NV_OK);
+        if (pMemdescData->hDupedPhysMem != 0)
+        {
+            NV_ASSERT(pRmApi->Free(pRmApi, pFabricVAS->hClient,
+                                   pMemdescData->hDupedPhysMem) == NV_OK);
+        }
 
         portMemFree(pMemDesc->_pMemData);
+        memdescSetMemData(pMemDesc, NULL, NULL);
     }
 
     portMemFree(pObject);
 }
 
-static NV_STATUS
-_memoryfabricMapPhysicalMemory
-(
-    NvU64             *vAddr,
-    NvU32              numAddr,
-    NvU64              allocSize,
-    NvU32              pageSize,
-    MEMORY_DESCRIPTOR *pVidMemDesc,
-    NvU64              offset,
-    NvBool             bReadOnly
-)
-{
-    OBJGPU                  *pGpu           = pVidMemDesc->pGpu;
-    VirtMemAllocator        *pDma           = GPU_GET_DMA(pGpu);
-    MemoryManager           *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
-    NV_STATUS                status         = NV_OK;
-    DMA_PAGE_ARRAY           pageArray;
-    NvU64                    mapLength;
-    NvU32                    kind;
-    COMPR_INFO               comprInfo;
-    NvU32                    vidmemPteArraySize;
-    RmPhysAddr               addr;
-    NvU32                    i;
-    FABRIC_VASPACE          *pFabricVAS;
-    NvU32                    mapFlags = DMA_UPDATE_VASPACE_FLAGS_UPDATE_ALL |
-                                        DMA_UPDATE_VASPACE_FLAGS_SKIP_4K_PTE_CHECK;
-
-    NV_ASSERT_OR_RETURN(vAddr != NULL,       NV_ERR_INVALID_ARGUMENT);
-    NV_ASSERT_OR_RETURN(pVidMemDesc != NULL, NV_ERR_INVALID_ARGUMENT);
-
-    mapFlags |= bReadOnly ? DMA_UPDATE_VASPACE_FLAGS_READ_ONLY : 0;
-
-    pFabricVAS = dynamicCast(pGpu->pFabricVAS, FABRIC_VASPACE);
-
-    if (pFabricVAS->bRpcAlloc)
-        return NV_OK;
-
-    // Get compression attributes for the vidmem memdesc.
-    status = memmgrGetKindComprFromMemDesc(pMemoryManager, pVidMemDesc, offset, &kind, &comprInfo);
-    if (status != NV_OK)
-    {
-        NV_PRINTF(LEVEL_ERROR,
-                  "Failed to get the compression attributes for the vidmem memdesc\n");
-        return status;
-    }
-
-    portMemSet(&pageArray, 0, sizeof(DMA_PAGE_ARRAY));
-
-    // Get the vidmem pteArray size.
-    vidmemPteArraySize = memdescGetPteArraySize(pVidMemDesc, AT_GPU);
-
-    // Get the fabric addr range to map.
-    mapLength       = (numAddr == 1 ? allocSize : pageSize);
-    pageArray.count = (vidmemPteArraySize == 1 ? 1 : (mapLength / RM_PAGE_SIZE));
-
-    for (i = 0; i < numAddr; i++)
-    {
-        if (pageArray.count == 1)
-        {
-            addr = pVidMemDesc->_pteArray[0] + offset;
-            pageArray.pData = &addr;
-        }
-        else
-        {
-            pageArray.pData = &pVidMemDesc->_pteArray[offset / RM_PAGE_SIZE];
-        }
-
-        // Map the memory fabric object at the given physical video memory offset.
-        status = dmaUpdateVASpace_HAL(pGpu, pDma, pFabricVAS->pGVAS, pVidMemDesc,
-                                      NULL, vAddr[i], vAddr[i] + mapLength - 1,
-                                      mapFlags, &pageArray, 0, &comprInfo, 0,
-                                      NV_MMU_PTE_VALID_TRUE,
-                                      NV_MMU_PTE_APERTURE_VIDEO_MEMORY,
-                                      BUS_INVALID_PEER, NVLINK_INVALID_FABRIC_ADDR,
-                                      DMA_DEFER_TLB_INVALIDATE, NV_FALSE);
-        if (status != NV_OK)
-        {
-            NV_PRINTF(LEVEL_ERROR,
-                      "Failed to map fabric addrs starting at 0x%llx\n", vAddr[i]);
-            return status;
-        }
-
-        offset = offset + mapLength;
-    }
-
-    fabricvaspaceInvalidateTlb(pFabricVAS, pVidMemDesc->pGpu, PTE_UPGRADE);
-
-    return NV_OK;
-}
-
 static void
 _memoryfabricFreeFabricVa_VGPU
 (
@@ -353,8 +476,6 @@ _memoryfabricAllocFabricVa
     NvU32                        *pNumAddr
 )
 {
-    OBJVASPACE *pOBJVASPACE = staticCast(pFabricVAS, OBJVASPACE);
-
     if (pFabricVAS->bRpcAlloc)
     {
         return _memoryfabricAllocFabricVa_VGPU(pGpu, pParams,
@@ -366,8 +487,8 @@ _memoryfabricAllocFabricVa
         return fabricvaspaceAllocNonContiguous(pFabricVAS,
                                                pAllocParams->allocSize,
                                                pAllocParams->alignment,
-                                               vaspaceGetVaStart(pOBJVASPACE),
-                                               vaspaceGetVaLimit(pOBJVASPACE),
+                                               fabricvaspaceGetUCFlaStart(pFabricVAS),
+                                               fabricvaspaceGetUCFlaLimit(pFabricVAS),
                                                pAllocParams->pageSize, flags,
                                                ppAddr, pNumAddr);
     }
@@ -387,8 +508,7 @@ memoryfabricConstruct_IMPL
     NV00F8_ALLOCATION_PARAMETERS *pAllocParams   = pParams->pAllocParams;
     RM_API                       *pRmApi         = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
     MemoryManager                *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
-    RsResourceRef                *pVidmemRef     = NULL;
-    MEMORY_DESCRIPTOR            *pVidMemDesc    = NULL;
+    MEMORY_DESCRIPTOR            *pPhysMemDesc    = NULL;
     NV_STATUS                     status         = NV_OK;
     MEMORY_DESCRIPTOR            *pMemDesc       = NULL;
     FABRIC_MEMDESC_DATA          *pMemdescData   = NULL;
@@ -398,6 +518,9 @@ memoryfabricConstruct_IMPL
     NvU32                         numAddr        = 0;
     NvU32                         pteKind        = 0;
     NvBool                        bReadOnly      = NV_FALSE;
+    NvHandle                      hPhysMem;
+    NvBool                        bFlexible      = NV_FALSE;
+    NvU32                         mapFlags       = 0;
 
     if (RS_IS_COPY_CTOR(pParams))
     {
@@ -406,6 +529,23 @@ memoryfabricConstruct_IMPL
                                               pParams);
     }
 
+    hPhysMem  = pAllocParams->map.hVidMem;
+
+    // Check if fabric vaspace is valid.
+    if (pFabricVAS == NULL)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Fabric vaspace object not available\n");
+
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
+    // initialize Fabric VAS Unicast range if not already setup 
+    if (fabricvaspaceGetUCFlaLimit(pFabricVAS) == 0)
+    {
+        NV_PRINTF(LEVEL_ERROR, "UC FLA ranges should be initialized by this time!\n");
+        return NV_ERR_INVALID_STATE;
+    }
+
     // Only page size 512MB and 2MB supported.
     if ((pAllocParams->pageSize != NV_MEMORY_FABRIC_PAGE_SIZE_512M) &&
         (pAllocParams->pageSize != NV_MEMORY_FABRIC_PAGE_SIZE_2M))
@@ -434,8 +574,10 @@ memoryfabricConstruct_IMPL
         return NV_ERR_INVALID_ARGUMENT;
     }
 
+    bFlexible = !!(pAllocParams->allocFlags & NV00F8_ALLOC_FLAGS_FLEXIBLE_FLA);
+
     // We don't support flexible mappings yet.
-    if (pAllocParams->allocFlags & NV00F8_ALLOC_FLAGS_FLEXIBLE_FLA)
+    if (bFlexible)
     {
         NV_PRINTF(LEVEL_ERROR,
                   "Only sticky mappings are supported\n");
@@ -445,7 +587,7 @@ memoryfabricConstruct_IMPL
 
     if (pAllocParams->allocFlags & NV00F8_ALLOC_FLAGS_READ_ONLY)
     {
-#if !defined(DEVELOP) && !defined(DEBUG) && !defined(NV_MODS)
+#if !defined(DEVELOP) && !defined(DEBUG) && !RMCFG_FEATURE_MODS_FEATURES
         NV_PRINTF(LEVEL_ERROR,
                   "RO mappings are only supported on non-release builds\n");
 
@@ -455,68 +597,20 @@ memoryfabricConstruct_IMPL
 #endif
     }
 
-    // For sticky mappings, physical video memory handle is needed.
-    if (pAllocParams->map.hVidMem == 0)
+    if (bFlexible && (hPhysMem != 0))
     {
         NV_PRINTF(LEVEL_ERROR,
-                  "Physical vidmem handle needed for sticky mappings\n");
-
-        return NV_ERR_INVALID_ARGUMENT;
-    }
-
-    status = serverutilGetResourceRef(pCallContext->pClient->hClient,
-                                      pAllocParams->map.hVidMem, &pVidmemRef);
-    if (status != NV_OK)
-    {
-        NV_PRINTF(LEVEL_ERROR,
-                  "Failed to get resource in resserv for vidmem handle\n");
-        return status;
-    }
-
-    pVidMemDesc = (dynamicCast(pVidmemRef->pResource, Memory))->pMemDesc;
-
-    if ((memdescGetAddressSpace(pVidMemDesc) != ADDR_FBMEM) ||
-        (pGpu != pVidMemDesc->pGpu))
-    {
-        NV_PRINTF(LEVEL_ERROR, "Invalid physical vidmem handle passed\n");
-
-        return NV_ERR_INVALID_ARGUMENT;
-    }
-
-    if (memdescGetPageSize(pVidMemDesc, AT_GPU) != NV_MEMORY_FABRIC_PAGE_SIZE_2M)
-    {
-        NV_PRINTF(LEVEL_ERROR, "Physical vidmem page size should be 2MB\n");
-
-        return NV_ERR_INVALID_ARGUMENT;
-    }
-
-    if ((pAllocParams->map.offset >= pVidMemDesc->Size) ||
-        !NV_IS_ALIGNED64(pAllocParams->map.offset, NV_MEMORY_FABRIC_PAGE_SIZE_2M))
-    {
-        NV_PRINTF(LEVEL_ERROR,
-                  "Invalid offset passed for the physical vidmem handle\n");
-
-        return NV_ERR_INVALID_OFFSET;
-    }
-
-    // hVidmem should be big enough to cover allocSize, starting from offset.
-    if (pAllocParams->allocSize >
-        (memdescGetSize(pVidMemDesc) - pAllocParams->map.offset))
-    {
-        NV_PRINTF(LEVEL_ERROR,
-                  "Insufficient physical video memory to map the requested "
-                  "memory fabric allocation\n");
-
-        return NV_ERR_INVALID_ARGUMENT;
-    }
-
-    // Check if fabric vaspace is valid.
-    if (pFabricVAS == NULL)
-    {
-        NV_PRINTF(LEVEL_ERROR, "Fabric vaspace object not available\n");
+                  "Physmem can't be provided during flexible object alloc\n");
 
         return NV_ERR_NOT_SUPPORTED;
     }
+    else if (!bFlexible)
+    {
+        status = _memoryfabricValidatePhysMem(pCallContext->pClient->hClient,
+                                              hPhysMem, pGpu, &pPhysMemDesc);
+        if (status != NV_OK)
+            return status;
+    }
 
     // Set the vaspace alloc flags.
     flags.bSkipTlbInvalidateOnFree = NV_TRUE;
@@ -536,8 +630,8 @@ memoryfabricConstruct_IMPL
                   "VA Space alloc failed! Status Code: 0x%x Size: 0x%llx "
                   "RangeLo: 0x%llx, RangeHi: 0x%llx, page size: 0x%x\n",
                   status, pAllocParams->allocSize,
-                  vaspaceGetVaStart(pGpu->pFabricVAS),
-                  vaspaceGetVaLimit(pGpu->pFabricVAS),
+                  fabricvaspaceGetUCFlaStart(pFabricVAS),
+                  fabricvaspaceGetUCFlaLimit(pFabricVAS),
                   pAllocParams->pageSize);
 
         return status;
@@ -617,19 +711,25 @@ memoryfabricConstruct_IMPL
 
     memdescAddDestroyCallback(pMemDesc, pCallback);
 
-    // Dup the physical video memory handle and cache it in memfabric memdesc.
-    status = pRmApi->DupObject(pRmApi, pFabricVAS->hClient, pFabricVAS->hDevice,
-                        &pMemdescData->hDupedVidmem, pCallContext->pClient->hClient,
-                        pAllocParams->map.hVidMem, 0);
-
-    if (status != NV_OK)
+    //
+    // In case of flexible mappings, we don't support:
+    //
+    // 1. Caching attributes of physical memory
+    // 2. FLA to GPA tracking to allow FLA object to be mapped as local memory
+    //
+    if (hPhysMem != 0)
     {
-        NV_PRINTF(LEVEL_ERROR, "Failed to dup physical video memory handle\n");
-        goto freeCallback;
-    }
+        // Dup the physical memory handle and cache it in memfabric memdesc.
+        status = pRmApi->DupObject(pRmApi, pFabricVAS->hClient, pFabricVAS->hDevice,
+                            &pMemdescData->hDupedPhysMem, pCallContext->pClient->hClient,
+                            hPhysMem, 0);
+
+        if (status != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR, "Failed to dup physmem handle\n");
+            goto freeCallback;
+        }
 
-    if (!(pAllocParams->allocFlags & NV00F8_ALLOC_FLAGS_FLEXIBLE_FLA))
-    {
         NV0041_CTRL_SURFACE_INFO surfaceInfo[2] = {0};
         NV0041_CTRL_GET_SURFACE_INFO_PARAMS surfaceInfoParam = {0};
 
@@ -640,34 +740,46 @@ memoryfabricConstruct_IMPL
 
         status = pRmApi->Control(pRmApi,
                                  pFabricVAS->hClient,
-                                 pMemdescData->hDupedVidmem,
+                                 pMemdescData->hDupedPhysMem,
                                  NV0041_CTRL_CMD_GET_SURFACE_INFO,
                                  &surfaceInfoParam,
                                  sizeof(surfaceInfoParam));
         if (status != NV_OK)
         {
-            NV_PRINTF(LEVEL_ERROR, "Failed to query physical video memory info\n");
+            NV_PRINTF(LEVEL_ERROR, "Failed to query physmem info\n");
             goto freeDupedMem;
         }
 
         pMemdescData->physAttrs.addressSpace = surfaceInfo[0].data;
         pMemdescData->physAttrs.compressionCoverage = surfaceInfo[1].data;
-    }
 
-    status = fabricvaspaceVaToGpaMapInsert(pFabricVAS, pAddr[0], pVidMemDesc,
-                                           pAllocParams->map.offset);
-    if (status != NV_OK)
-        goto freeDupedMem;
+        mapFlags |= bReadOnly ? FABRIC_VASPACE_MAP_FLAGS_READ_ONLY : 0;
 
-    // Map the memory fabric object at the given physical video memory offset.
-    status = _memoryfabricMapPhysicalMemory(pAddr, numAddr, pAllocParams->allocSize,
-                                            pAllocParams->pageSize, pVidMemDesc,
-                                            pAllocParams->map.offset, bReadOnly);
-    if (status != NV_OK)
-    {
-        NV_PRINTF(LEVEL_ERROR,
-                "Failed to map FLA at the given physical vidmem offset\n");
-        goto memFabricRemoveVaToGpaMap;
+        //
+        // Sticky FLA object should be mapped completely, so pass
+        // pAllocParams->allocSize.as mapLength.
+        //
+        status = fabricvaspaceMapPhysMemdesc(pFabricVAS,
+                                             pMemDesc, 0,
+                                             pPhysMemDesc,
+                                             pAllocParams->map.offset,
+                                             pAllocParams->allocSize,
+                                             mapFlags);
+        if (status != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR,
+                    "Failed to map FLA at the given physmem offset\n");
+            goto freeDupedMem;
+        }
+
+        //
+        // No need to unmap on failure. Unmap happens implicitly when fabric VA
+        // would be freed.
+        //
+        status = fabricvaspaceVaToGpaMapInsert(pFabricVAS, pAddr[0], pPhysMemDesc,
+                                               pAllocParams->map.offset);
+        if (status != NV_OK)
+            goto freeDupedMem;
     }
 
     pMemdescData->allocFlags = pAllocParams->allocFlags;
@@ -677,13 +789,10 @@ memoryfabricConstruct_IMPL
 
     return NV_OK;
 
-memFabricRemoveVaToGpaMap:
-    fabricvaspaceVaToGpaMapRemove(pFabricVAS, pAddr[0]);
-
 freeDupedMem:
-    // Free the duped vidmem handle.
+    // Free the duped physmem handle.
     NV_ASSERT(pRmApi->Free(pRmApi, pFabricVAS->hClient,
-                           pMemdescData->hDupedVidmem) == NV_OK);
+                           pMemdescData->hDupedPhysMem) == NV_OK);
 
 freeCallback:
     // Destroy the memdesc destroy callback.
@@ -754,30 +863,6 @@ memoryfabricCopyConstruct_IMPL
     return NV_OK;
 }
 
-NvBool
-memoryfabricCanExport_IMPL
-(
-    MemoryFabric *pMemoryFabric
-)
-{
-    Memory *pMemory = staticCast(pMemoryFabric, Memory);
-    FABRIC_MEMDESC_DATA *pMemdescData;
-
-    if (pMemory->pMemDesc == NULL)
-        return NV_ERR_INVALID_ARGUMENT;
-
-    pMemdescData = (FABRIC_MEMDESC_DATA *)memdescGetMemData(pMemory->pMemDesc);
-
-    //
-    // Check if FLA->PA mappings are present. Only then allow export.
-    // FLA->PA mappings are guaranteed for STICKY FLA mappings, which is only
-    // what we support currently.
-    // TODO: Re-visit this function when support for FLEXIBLE FLA mappings is
-    // added.
-    //
-    return !(pMemdescData->allocFlags & NV00F8_ALLOC_FLAGS_FLEXIBLE_FLA);
-}
-
 NV_STATUS
 memoryfabricControl_IMPL
 (
@@ -827,5 +912,111 @@ memoryfabricCtrlCmdDescribe_IMPL
     NV00F8_CTRL_DESCRIBE_PARAMS   *pParams
 )
 {
-    return NV_ERR_NOT_SUPPORTED;
+    Memory *pMemory = staticCast(pMemoryFabric, Memory);
+    NvU64  *pFabricArray;
+    NvU64   offset;
+    NvU64   pageSize;
+    NvU32   i;
+
+    if (pMemory->pMemDesc == NULL)
+        return NV_ERR_INVALID_ARGUMENT;
+
+    pageSize = memdescGetPageSize(pMemory->pMemDesc, AT_GPU);
+
+    if (memdescGetContiguity(pMemory->pMemDesc, AT_GPU))
+        pParams->totalPfns = 1;
+    else
+        pParams->totalPfns = memdescGetSize(pMemory->pMemDesc) / pageSize;
+
+    if (pParams->offset >= pParams->totalPfns)
+    {
+        NV_PRINTF(LEVEL_ERROR, "offset: %llx is out of range: %llx \n", pParams->offset,
+                  pParams->totalPfns);
+        return NV_ERR_OUT_OF_RANGE;
+    }
+
+    pParams->numPfns = NV_MIN(pParams->totalPfns - pParams->offset,
+                              NV00F8_CTRL_DESCRIBE_PFN_ARRAY_SIZE);
+
+    pFabricArray = portMemAllocNonPaged(sizeof(NvU64) * pParams->numPfns);
+
+    if (pFabricArray == NULL)
+        return NV_ERR_NO_MEMORY;
+
+    offset = pParams->offset * pageSize;
+    memdescGetPhysAddrsForGpu(pMemory->pMemDesc, pMemory->pMemDesc->pGpu, AT_GPU, offset,
+                              pageSize, pParams->numPfns,
+                              pFabricArray);
+
+    for (i = 0; i < pParams->numPfns; i++)
+    {
+        pParams->pfnArray[i] = (NvU32)(pFabricArray[i] >> RM_PAGE_SHIFT_HUGE);
+    }
+
+    portMemFree(pFabricArray);
+
+    return NV_OK;
+}
+
+NV_STATUS
+memoryfabricCtrlAttachMem_IMPL
+(
+    MemoryFabric                  *pMemoryFabric,
+    NV00F8_CTRL_ATTACH_MEM_PARAMS *pParams
+)
+{
+    NvU32 i;
+    NV_STATUS status;
+
+    pParams->numAttached = 0;
+
+    if (pParams->flags != 0)
+        return NV_ERR_INVALID_ARGUMENT;
+
+    if ((pParams->numMemInfos == 0) ||
+        (pParams->numMemInfos > NV00F8_MAX_ATTACHABLE_MEM_INFOS))
+        return NV_ERR_INVALID_ARGUMENT;
+
+    for (i = 0; i < pParams->numMemInfos; i++)
+    {
+        status = _memoryFabricAttachMem(pMemoryFabric, &pParams->memInfos[i]);
+        if (status != NV_OK)
+            return status;
+
+        pParams->numAttached++;
+    }
+
+    return NV_OK;
+}
+
+NV_STATUS
+memoryfabricCtrlDetachMem_IMPL
+(
+    MemoryFabric                  *pMemoryFabric,
+    NV00F8_CTRL_DETACH_MEM_PARAMS *pParams
+)
+{
+    NvU32 i;
+    NV_STATUS status;
+    Memory *pMemory = staticCast(pMemoryFabric, Memory);
+
+    pParams->numDetached = 0;
+
+    if (pParams->flags != 0)
+        return NV_ERR_INVALID_ARGUMENT;
+
+    if ((pParams->numOffsets == 0) ||
+         pParams->numOffsets > NV00F8_MAX_DETACHABLE_OFFSETS)
+        return NV_ERR_INVALID_ARGUMENT;
+
+    for (i = 0; i < pParams->numOffsets; i++)
+    {
+        status = _memoryFabricDetachMem(pMemory->pMemDesc, pParams->offsets[i]);
+        if (status != NV_OK)
+            return status;
+
+        pParams->numDetached++;
+    }
+
+    return NV_OK;
 }
