@@ -22,7 +22,7 @@
  */
 
 
-/***************************** HW State Rotuines ***************************\
+/***************************** HW State Routines ***************************\
 *         Core Logic Object Function Definitions.                           *
 \***************************************************************************/
 
@@ -133,6 +133,9 @@ clInitMappingPciBusDevice_IMPL
     NvU16 vendorID, deviceID;
     NvBool bFoundDevice = NV_FALSE;
 
+    if (IsT194(pGpu))
+        return NV0000_CTRL_GPU_INVALID_ID;
+
     // do we already know our domain/bus/device?
     if (gpuGetDBDF(pGpu) == 0)
     {
@@ -791,5 +794,6 @@ void clSyncWithGsp_IMPL(OBJCL *pCl, GspSystemInfo *pGSI)
 
     pGSI->Chipset = pCl->Chipset;
     pGSI->FHBBusInfo = pCl->FHBBusInfo;
+    pGSI->chipsetIDInfo = pCl->chipsetIDInfo;
 
 }
