@@ -34,16 +34,21 @@
 #include "resource_desc.h"
 #include "gpu_mgr/gpu_mgr.h"
 #include "gpu/gpu.h"
+#include "gpu/mmu/kern_gmmu.h"
 
 #include "gpu/bus/third_party_p2p.h"
+#include "virtualization/hypervisor/hypervisor.h"
 
+OsInfoMap g_osInfoList;
 UserInfoList g_userInfoList;
 RmClientList g_clientListBehindGpusLock; // RS-TODO remove this WAR
 
 #define RS_FW_UNIQUE_HANDLE_BASE  (0xc9f00000)
 
-NV_STATUS _registerUserInfo(PUID_TOKEN *ppUidToken, UserInfo **ppUserInfo);
-NV_STATUS _unregisterUserInfo(UserInfo *pUserInfo);
+static NV_STATUS _registerUserInfo(PUID_TOKEN *ppUidToken, UserInfo **ppUserInfo);
+static NV_STATUS _unregisterUserInfo(UserInfo *pUserInfo);
+static NV_STATUS _registerOSInfo(RmClient *pClient, void *pOSInfo);
+static NV_STATUS _unregisterOSInfo(RmClient *pClient, void *pOSInfo);
 
 NV_STATUS
 rmclientConstruct_IMPL
@@ -59,13 +64,40 @@ rmclientConstruct_IMPL
     RsClient          *pRsClient = staticCast(pClient, RsClient);
     NvBool             bReleaseLock = NV_FALSE;
     API_SECURITY_INFO *pSecInfo = pParams->pSecInfo;
+    OBJGPU            *pGpu = NULL;
+
+    if (RMCFG_FEATURE_PLATFORM_GSP)
+    {
+        pGpu = gpumgrGetSomeGpu();
+
+        if (pGpu == NULL)
+        {
+            NV_PRINTF(LEVEL_ERROR, "GPU is not found\n");
+            return NV_ERR_INVALID_STATE;
+        }
+    }
 
     pClient->bIsRootNonPriv  = (pParams->externalClassId == NV01_ROOT_NON_PRIV);
-    pClient->ProcID          = osGetCurrentProcess();
     pClient->pUserInfo       = NULL;
     pClient->pSecurityToken  = NULL;
     pClient->pOSInfo         = pSecInfo->clientOSInfo;
 
+    // TODO: Revisit in M2, see GPUSWSEC-1176
+    if (RMCFG_FEATURE_PLATFORM_GSP && IS_VGPU_GSP_PLUGIN_OFFLOAD_ENABLED(pGpu))
+    {
+        if (pSecInfo->pProcessToken != NULL && ((NvU64) pSecInfo->pProcessToken) < VMMU_MAX_GFID)
+        {
+            // Trunc to NvU32 to fit ProcID (VMMU_MAX_GFID << MAX_INT)
+            pClient->ProcID = (NvU32)((NvU64)pSecInfo->pProcessToken);
+
+            NV_PRINTF(LEVEL_INFO, "Client allocation with GFID = %u\n", (NvU32)((NvU64)pSecInfo->pProcessToken));
+        }
+    }
+    else
+    {
+        pClient->ProcID = osGetCurrentProcess();
+    }
+
     pClient->cachedPrivilege = pSecInfo->privLevel;
 
     // Set user-friendly client name from current process
@@ -77,6 +109,29 @@ rmclientConstruct_IMPL
             NV0000_CTRL_EVENT_SET_NOTIFICATION_ACTION_DISABLE;
     }
 
+    //
+    // Enabling this on MODS to avoid clash of client handles. This path gets executed on both
+    // guest & host RM for MODs platform, pGPU handle isnt available here to check for IS_VIRTUAL.
+    // Later code paths will override this for guest RM.
+    // This change affects non-SRIOV case as well, there is no good way to detect SRIOV without pGPU.
+    //
+    if (hypervisorIsVgxHyper() || NV_IS_MODS)
+    {
+        //
+        // Set RM allocated resource handle range for host RM. This minimize clash of guest RM handles with host RM
+        // during VM migration.
+        //
+        status = clientSetHandleGenerator(pRsClient,
+                                          (RS_UNIQUE_HANDLE_BASE + RS_UNIQUE_HANDLE_RANGE/2),
+                                          RS_UNIQUE_HANDLE_RANGE/2);
+        if (status != NV_OK)
+        {
+            NV_PRINTF(LEVEL_WARNING,
+                      "NVRM_RPC: Failed to set host client resource handle range %x\n", status);
+            return status;
+        }
+    }
+
     // Prevent kernel clients from requesting handles in the FW handle generator range
     status = clientSetRestrictedRange(pRsClient,
                                       RS_FW_UNIQUE_HANDLE_BASE, RS_UNIQUE_HANDLE_RANGE);
@@ -98,6 +153,8 @@ rmclientConstruct_IMPL
         bReleaseLock = NV_TRUE;
     }
 
+    _registerOSInfo(pClient, pClient->pOSInfo);
+
     pClient->bIsClientVirtualMode = (pSecInfo->pProcessToken != NULL);
 
     //
@@ -107,27 +164,32 @@ rmclientConstruct_IMPL
     if (pSys->getProperty(pSys, PDB_PROP_SYS_VALIDATE_CLIENT_HANDLE) &&
        ((pParams->pSecInfo->privLevel < RS_PRIV_LEVEL_KERNEL) || pClient->bIsRootNonPriv))
     {
-        PSECURITY_TOKEN pSecurityToken;
+        PSECURITY_TOKEN pSecurityToken = (pClient->bIsClientVirtualMode ?
+                                          pSecInfo->pProcessToken : osGetSecurityToken());
         PUID_TOKEN pUidToken = osGetCurrentUidToken();
         UserInfo *pUserInfo = NULL;
 
-        pSecurityToken  = (pClient->bIsClientVirtualMode ?
-                           pSecInfo->pProcessToken : osGetSecurityToken());
-
-        // pUserInfo takes ownership of pUidToken upon successful registration
-        status = _registerUserInfo(&pUidToken, &pUserInfo);
-
-        if (status == NV_OK)
+        if (RMCFG_FEATURE_PLATFORM_GSP && IS_VGPU_GSP_PLUGIN_OFFLOAD_ENABLED(pGpu))
         {
-            pClient->pUserInfo = pUserInfo;
             pClient->pSecurityToken = pSecurityToken;
         }
         else
         {
-            portMemFree(pUidToken);
+            // pUserInfo takes ownership of pUidToken upon successful registration
+            status = _registerUserInfo(&pUidToken, &pUserInfo);
 
-            if (pSecurityToken != NULL && !pClient->bIsClientVirtualMode)
-                portMemFree(pSecurityToken);
+            if (status == NV_OK)
+            {
+                pClient->pUserInfo = pUserInfo;
+                pClient->pSecurityToken = pSecurityToken;
+            }
+            else
+            {
+                portMemFree(pUidToken);
+
+                if (pSecurityToken != NULL && !pClient->bIsClientVirtualMode)
+                    portMemFree(pSecurityToken);
+            }
         }
     }
 
@@ -168,8 +230,6 @@ rmclientDestruct_IMPL
     // Free any association of the client with existing third-party p2p object
     CliUnregisterFromThirdPartyP2P(pClient);
 
-    rmapiControlCacheFreeClient(hClient);
-
     //
     // Free all of the devices of the client (do it in reverse order to
     // facilitate tear down of things like ctxdmas, etc)
@@ -207,6 +267,8 @@ rmclientDestruct_IMPL
         }
     }
 
+    _unregisterOSInfo(pClient, pClient->pOSInfo);
+
     listRemoveFirstByValue(&g_clientListBehindGpusLock, (void*)&pClient);
 
     if (pClient->pUserInfo != NULL)
@@ -567,7 +629,7 @@ NV_STATUS rmclientUserClientSecurityCheckByHandle(NvHandle hClient, const API_SE
  * @param[inout] ppUidToken
  * @param[out] ppUserInfo
  */
-NV_STATUS
+static NV_STATUS
 _registerUserInfo
 (
     PUID_TOKEN *ppUidToken,
@@ -632,7 +694,7 @@ _registerUserInfo
  *
  * @param[in] pUserInfo
  */
-NV_STATUS
+static NV_STATUS
 _unregisterUserInfo
 (
     UserInfo *pUserInfo
@@ -834,3 +896,77 @@ NvBool rmclientIsCapableByHandle
 
     return rmclientIsCapable(pClient, capability);
 }
+
+/**
+ *
+ * Register a client with a user info list
+ *
+ * This function must be protected by a lock (currently the GPUs lock.)
+ *
+ * @param[in] pClient
+ * @param[in] pOSInfo
+ */
+static NV_STATUS
+_registerOSInfo
+(
+   RmClient *pClient,
+   void *pOSInfo
+)
+{
+    OsInfoMapSubmap *pSubmap = NULL;
+    RmClient **pInsert = NULL;
+    NvU64 key1 = (NvUPtr)pOSInfo;
+    NvU64 key2 = (NvU64)(staticCast(pClient,RsClient))->hClient;
+
+    if (multimapFindItem(&g_osInfoList, key1, key2) != NULL)
+        return NV_ERR_INSERT_DUPLICATE_NAME;
+
+    if (multimapFindSubmap(&g_osInfoList, key1) == NULL)
+    {
+        pSubmap = multimapInsertSubmap(&g_osInfoList, key1);
+        if (pSubmap == NULL)
+            return NV_ERR_NO_MEMORY;
+    }
+
+    pInsert = multimapInsertItemNew(&g_osInfoList, key1, key2);
+    if (pInsert == NULL)
+        return NV_ERR_NO_MEMORY;
+
+    *pInsert = pClient;
+
+    return NV_OK;
+}
+
+/**
+ *
+ * Unregister a client from a user info list
+ *
+ * This function must be protected by a lock (currently the GPUs lock.)
+ *
+ * @param[in] pClient
+ * @param[in] pOSInfo
+ */
+static NV_STATUS
+_unregisterOSInfo
+(
+    RmClient *pClient,
+    void *pOSInfo
+)
+{
+     NvU64 key1 = (NvUPtr)pOSInfo;
+     NvU64 key2 = (NvU64)(staticCast(pClient, RsClient))->hClient;
+     OsInfoMapSubmap *pSubmap = NULL;
+     RmClient **pFind = NULL;
+ 
+     pFind = multimapFindItem(&g_osInfoList, key1, key2);
+     if (pFind != NULL)
+         multimapRemoveItem(&g_osInfoList, pFind);
+
+     pSubmap = multimapFindSubmap(&g_osInfoList, key1);
+     if (pSubmap == NULL || multimapCountSubmapItems(&g_osInfoList, pSubmap) > 0)
+         return NV_OK;
+ 
+     multimapRemoveSubmap(&g_osInfoList, pSubmap);
+ 
+     return NV_OK;
+}
